/**
 * 
 */
package com.github.ansell.oas.website;

import org.restlet.Component;
import org.restlet.data.LocalReference;
import org.restlet.data.Parameter;
import org.restlet.data.Protocol;
import org.restlet.data.Reference;
import org.restlet.representation.Representation;
import org.restlet.routing.Router;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.bridge.SLF4JBridgeHandler;

import com.github.ansell.oas.utils.ApplicationUtils;
import com.github.ansell.oas.webservice.OasWebServiceApplication;
import com.github.ansell.oas.webservice.impl.OasWebServiceApplicationImpl;
import com.github.ansell.restletutils.ClassLoaderDirectory;
import com.github.ansell.restletutils.CompositeClassLoader;

/**
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public class OASWebsiteComponent extends Component
{
    private final Logger log = LoggerFactory.getLogger(this.getClass());
    private String resetKey;
    
    static
    {
        // add the ability to view java.util.logging messages using SLF4J
        System.getProperties().put("org.restlet.engine.loggerFacadeClass", "org.restlet.ext.slf4j.Slf4jLoggerFacade");
        SLF4JBridgeHandler.install();
    }
    
    /**
     * 
     */
    public OASWebsiteComponent()
    {
        super();
        
        this.getClients().add(Protocol.CLAP);
        this.getClients().add(Protocol.HTTP);
        // this.initialise();
    }
    
    /**
     * @param arg0
     */
    public OASWebsiteComponent(final Reference arg0)
    {
        super(arg0);
        
        this.getClients().add(Protocol.CLAP);
        this.getClients().add(Protocol.HTTP);
        // this.initialise();
    }
    
    /**
     * @param xmlConfigRepresentation
     */
    public OASWebsiteComponent(final Representation xmlConfigRepresentation)
    {
        super(xmlConfigRepresentation);
        
        this.getClients().add(Protocol.CLAP);
        this.getClients().add(Protocol.HTTP);
        // this.initialise();
    }
    
    /**
     * @param xmlConfigurationRef
     */
    public OASWebsiteComponent(final String xmlConfigurationRef)
    {
        super(xmlConfigurationRef);
        
        this.getClients().add(Protocol.CLAP);
        this.getClients().add(Protocol.HTTP);
        // this.initialise();
    }
    
    /**
     * @return the resetKey
     */
    protected String getResetKey()
    {
        return this.resetKey;
    }
    
    @Override
    public synchronized void start() throws Exception
    {
        super.start();
        
        String propertiesBundleName = "oas";
        
        this.log.debug("Dumping component context attributes...");
        for(String parameter : getContext().getAttributes().keySet())
        {
            this.log.trace("component context attribute: <{}> <{}>", parameter,
             getContext().getAttributes().get(parameter));
            if(parameter.equalsIgnoreCase("oasproperties"))
            {
                Object value = getContext().getAttributes().get(parameter);
                
                if(value instanceof String)
                {
                    String stringValue = (String)value;
                    // If the string value starts with this then it hasn't been replaced by maven,
                    // so ignore it
                    if(!stringValue.startsWith("${"))
                    {
                        this.log.info("Found context attribute defining oasproperties: {}", stringValue);
                        propertiesBundleName = stringValue;
                    }
                }
            }
        }
        
        this.log.debug("Dumping component context parameters...");
        for(Parameter parameter : getContext().getParameters())
        {
            this.log.trace("component context parameter: <{}> <{}>", parameter.getName(), parameter.getValue());
            
            if(parameter.getName().equalsIgnoreCase("oasproperties"))
            {
                // If the string value starts with this then it hasn't been replaced by maven, so
                // ignore it
                if(!parameter.getValue().startsWith("${"))
                {
                    this.log.info("Found context attribute defining oasproperties: {}", parameter.getValue());
                    propertiesBundleName = parameter.getValue();
                }
            }
        }
        
        this.log.debug("Creating application...");
        // NOTE: This needs to be Impl as Restlet is not designed using interfaces, so we cannot
        // easily include our interface in the process.
        final OasWebServiceApplication nextApplication =
                new OasWebServiceApplicationImpl(this.getContext().createChildContext(), propertiesBundleName);
        
        this.log.debug("Attaching application...");
        // attach the web services application
        this.getDefaultHost().attach("/", nextApplication);
        
        this.log.debug("Setting up application...");
        // setup the application after attaching it, as it requires Application.getContext() to not
        // be null during the setup process
        ApplicationUtils.setupApplication(nextApplication, nextApplication.getContext());
        
        // FIXME: Make this configurable
        final LocalReference localReference = LocalReference.createClapReference(LocalReference.CLAP_THREAD, "static/");
        
        final CompositeClassLoader customClassLoader = new CompositeClassLoader();
        customClassLoader.addClassLoader(Thread.currentThread().getContextClassLoader());
        customClassLoader.addClassLoader(Router.class.getClassLoader());
        
        final ClassLoaderDirectory directory =
                new ClassLoaderDirectory(this.getContext().createChildContext(), localReference, customClassLoader);
        
        directory.setListingAllowed(true);
        
        final String resourcesPath = "/resources/";
        
        this.log.debug("attaching resource handler to path={}", resourcesPath);
        
        // attach the resources directory
        this.getDefaultHost().attach(resourcesPath, directory);
        
        // Create a resource to use in integration tests to reset the application to a fresh state
        // on demand, without adding this functionality to OasWebServiceApplicationImpl
        final TestResetResource reset = new TestResetResource(nextApplication);
        this.setResetKey(nextApplication.getPropertyUtil().get("oas.test.website.reset.key", ""));
        final String resetPath = "/reset/" + this.getResetKey();
        this.getDefaultHost().attach(resetPath, reset);
        
        this.log.info("routes={}", this.getDefaultHost().getRoutes().toString());
        
    }
    
    /**
     * This field is used in testing to enable the resetting of the internal elements of the website
     * after each test.
     * 
     * It is protected by a simple runtime generated key to prevent this function leaking out if
     * people directly deploy this TEST website component.
     * 
     * @param key
     *            A simple key shared with us by the test running environment.
     */
    protected void setResetKey(final String key)
    {
        this.resetKey = key;
    }
}
